@@ -18,16 +18,17 @@
 
 namespace ruckig {
 
-constexpr static size_t DynamicDOFs {0};
-
 //! Main class for the Ruckig algorithm.
-template<size_t DOFs = 0, bool throw_error = false, bool return_error_at_maximal_duration = true>
+template<size_t DOFs = 0, template<class, size_t> class CustomVector = StandardVector, bool throw_error = false>
 class Ruckig {
     //! Current input, only for comparison for recalculation
-    InputParameter<DOFs> current_input;
+    InputParameter<DOFs, CustomVector> current_input;
+
+    //! Flag that indicates if the current_input was properly initialized
+    bool current_input_initialized {false};
 
     //! Calculator for new trajectories
-    Calculator<DOFs> calculator;
+    Calculator<DOFs, CustomVector> calculator;
 
     //! Max number of intermediate waypoints
     const size_t max_number_of_waypoints;
@@ -37,42 +38,47 @@ class Ruckig {
     }
 
 public:
-    size_t degrees_of_freedom;
+    const size_t degrees_of_freedom;
 
     //! Time step between updates (cycle time) in [s]
-    const double delta_time;
+    double delta_time {0.0};
 
     template <size_t D = DOFs, typename std::enable_if<D >= 1, int>::type = 0>
-    explicit Ruckig(): degrees_of_freedom(DOFs), delta_time(-1.0), max_number_of_waypoints(0) {
+    explicit Ruckig(): max_number_of_waypoints(0), degrees_of_freedom(DOFs), delta_time(-1.0) {
     }
 
     template <size_t D = DOFs, typename std::enable_if<D >= 1, int>::type = 0>
-    explicit Ruckig(double delta_time): degrees_of_freedom(DOFs), delta_time(delta_time), max_number_of_waypoints(0) {
+    explicit Ruckig(double delta_time): max_number_of_waypoints(0), degrees_of_freedom(DOFs), delta_time(delta_time) {
     }
 
 #if defined WITH_ONLINE_CLIENT
     template <size_t D = DOFs, typename std::enable_if<D >= 1, int>::type = 0>
-    explicit Ruckig(double delta_time, size_t max_number_of_waypoints): degrees_of_freedom(DOFs), delta_time(delta_time), calculator(Calculator<DOFs>(max_number_of_waypoints)), max_number_of_waypoints(max_number_of_waypoints), current_input(InputParameter<DOFs>(max_number_of_waypoints)) {
+    explicit Ruckig(double delta_time, size_t max_number_of_waypoints): current_input(InputParameter<DOFs, CustomVector>(max_number_of_waypoints)), calculator(Calculator<DOFs, CustomVector>(max_number_of_waypoints)), max_number_of_waypoints(max_number_of_waypoints), degrees_of_freedom(DOFs), delta_time(delta_time) {
     }
 #endif
 
     template <size_t D = DOFs, typename std::enable_if<D == 0, int>::type = 0>
-    explicit Ruckig(size_t dofs): degrees_of_freedom(dofs), delta_time(-1.0), calculator(Calculator<0>(dofs)), max_number_of_waypoints(0), current_input(InputParameter<0>(dofs)) {
+    explicit Ruckig(size_t dofs): current_input(InputParameter<DOFs, CustomVector>(dofs)), calculator(Calculator<DOFs, CustomVector>(dofs)), max_number_of_waypoints(0), degrees_of_freedom(dofs), delta_time(-1.0) {
     }
 
     template <size_t D = DOFs, typename std::enable_if<D == 0, int>::type = 0>
-    explicit Ruckig(size_t dofs, double delta_time): degrees_of_freedom(dofs), delta_time(delta_time), calculator(Calculator<0>(dofs)), max_number_of_waypoints(0), current_input(InputParameter<0>(dofs)) {
+    explicit Ruckig(size_t dofs, double delta_time): current_input(InputParameter<DOFs, CustomVector>(dofs)), calculator(Calculator<DOFs, CustomVector>(dofs)), max_number_of_waypoints(0), degrees_of_freedom(dofs), delta_time(delta_time) {
     }
 
 #if defined WITH_ONLINE_CLIENT
     template <size_t D = DOFs, typename std::enable_if<D == 0, int>::type = 0>
-    explicit Ruckig(size_t dofs, double delta_time, size_t max_number_of_waypoints): degrees_of_freedom(dofs), delta_time(delta_time), calculator(Calculator<0>(dofs, max_number_of_waypoints)), max_number_of_waypoints(max_number_of_waypoints), current_input(InputParameter<0>(dofs, max_number_of_waypoints)) {
+    explicit Ruckig(size_t dofs, double delta_time, size_t max_number_of_waypoints): current_input(InputParameter<DOFs, CustomVector>(dofs, max_number_of_waypoints)), calculator(Calculator<DOFs, CustomVector>(dofs, max_number_of_waypoints)), max_number_of_waypoints(max_number_of_waypoints), degrees_of_freedom(dofs), delta_time(delta_time) {
     }
 #endif
 
+    //! Reset the instance (e.g. to force a new calculation in the next update)
+    void reset() {
+        current_input_initialized = false;
+    }
+
     //! Filter intermediate positions based on a threshold distance for each DoF
-    template<class T> using Vector = typename std::conditional<DOFs >= 1, std::array<T, DOFs>, std::vector<T>>::type;
-    std::vector<Vector<double>> filter_intermediate_positions(const InputParameter<DOFs>& input, const Vector<double>& threshold_distance) const {
+    template<class T> using Vector = CustomVector<T, DOFs>;
+    std::vector<Vector<double>> filter_intermediate_positions(const InputParameter<DOFs, CustomVector>& input, const Vector<double>& threshold_distance) const {
         if (input.intermediate_positions.empty()) {
             return input.intermediate_positions;
         }
@@ -134,7 +140,7 @@ public:
     }
 
     //! Validate the input for trajectory calculation and kinematic limits
-    bool validate_input(const InputParameter<DOFs>& input, bool check_current_state_within_limits=false, bool check_target_state_within_limits=true) const {
+    bool validate_input(const InputParameter<DOFs, CustomVector>& input, bool check_current_state_within_limits=false, bool check_target_state_within_limits=true) const {
         for (size_t dof = 0; dof < degrees_of_freedom; ++dof) {
             const double jMax = input.max_jerk[dof];
             if (std::isnan(jMax) || jMax <= std::numeric_limits<double>::min()) {
@@ -172,16 +178,19 @@ public:
 
             auto control_interface = input.per_dof_control_interface ? input.per_dof_control_interface.value()[dof] : input.control_interface;
             if (control_interface == ControlInterface::Position) {
-                if (std::isnan(input.current_position[dof]) || std::isnan(input.target_position[dof])) {
+                const double p0 = input.current_position[dof];
+                const double pf = input.target_position[dof];
+                if (std::isnan(p0) || std::isnan(pf)) {
                     return false;
                 }
 
                 const double vMax = input.max_velocity[dof];
+                const double vMin = input.min_velocity ? input.min_velocity.value()[dof] : -input.max_velocity[dof];
+
                 if (std::isnan(vMax) || vMax <= std::numeric_limits<double>::min()) {
                     return false;
                 }
 
-                const double vMin = input.min_velocity ? input.min_velocity.value()[dof] : -input.max_velocity[dof];
                 if (std::isnan(vMin) || vMin >= -std::numeric_limits<double>::min()) {
                     return false;
                 }
@@ -216,27 +225,31 @@ public:
             }
         }
 
+        if (delta_time <= 0.0 && input.duration_discretization != DurationDiscretization::Continuous) {
+            return false;
+        }
+
         return true;
     }
 
     //! Calculate a new trajectory for the given input
-    Result calculate(const InputParameter<DOFs>& input, Trajectory<DOFs>& trajectory) {
+    Result calculate(const InputParameter<DOFs, CustomVector>& input, Trajectory<DOFs, CustomVector>& trajectory) {
         bool was_interrupted {false};
         return calculate(input, trajectory, was_interrupted);
     }
 
     //! Calculate a new trajectory for the given input and check for interruption
-    Result calculate(const InputParameter<DOFs>& input, Trajectory<DOFs>& trajectory, bool& was_interrupted) {
+    Result calculate(const InputParameter<DOFs, CustomVector>& input, Trajectory<DOFs, CustomVector>& trajectory, bool& was_interrupted) {
         if (!validate_input(input, false, true)) {
             return Result::ErrorInvalidInput;
         }
 
-        return calculator.template calculate<throw_error, return_error_at_maximal_duration>(input, trajectory, delta_time, was_interrupted);
+        return calculator.template calculate<throw_error>(input, trajectory, delta_time, was_interrupted);
     }
 
     //! Get the next output state (with step delta_time) along the calculated trajectory for the given input
-    Result update(const InputParameter<DOFs>& input, OutputParameter<DOFs>& output) {
-        const auto start = std::chrono::high_resolution_clock::now();
+    Result update(const InputParameter<DOFs, CustomVector>& input, OutputParameter<DOFs, CustomVector>& output) {
+        const auto start = std::chrono::steady_clock::now();
 
         if constexpr (DOFs == 0 && throw_error) {
             if (degrees_of_freedom != input.degrees_of_freedom || degrees_of_freedom != output.degrees_of_freedom) {
@@ -246,13 +259,15 @@ public:
 
         output.new_calculation = false;
 
-        if (input != current_input) {
-            Result result = calculate(input, output.trajectory, output.was_calculation_interrupted);
-            if (result != Result::Working) {
+        Result result {Result::Working};
+        if (input != current_input || !current_input_initialized) {
+            result = calculate(input, output.trajectory, output.was_calculation_interrupted);
+            if (result != Result::Working && result != Result::ErrorPositionalLimits) {
                 return result;
             }
 
             current_input = input;
+            current_input_initialized = true;
             output.time = 0.0;
             output.new_calculation = true;
         }
@@ -262,7 +277,7 @@ public:
         output.trajectory.at_time(output.time, output.new_position, output.new_velocity, output.new_acceleration, output.new_section);
         output.did_section_change = (output.new_section != old_section);
 
-        const auto stop = std::chrono::high_resolution_clock::now();
+        const auto stop = std::chrono::steady_clock::now();
         output.calculation_duration = std::chrono::duration_cast<std::chrono::nanoseconds>(stop - start).count() / 1000.0;
 
         output.pass_to_input(current_input);
@@ -271,8 +286,13 @@ public:
             return Result::Finished;
         }
 
-        return Result::Working;
+        return result;
     }
 };
 
+
+template<size_t DOFs, template<class, size_t> class CustomVector = StandardVector>
+using RuckigThrow = Ruckig<DOFs, CustomVector, true>;
+
+
 } // namespace ruckig
